;;; $Id: my-3-5-dynamic.scm,v 1.1 2005/03/21 05:15:02 leavens Exp leavens $

(displayln "Name:  Gary T. Leavens")
(displayln "Section: all")

(let ((time-stamp "Time-stamp: <2001-05-10 16:12:14 dfried>"))
  (eopl:printf "3-5.scm: language with procedures ~a~%"
    (substring time-stamp 13 29)))

;;;;;;;;;;;;;;;; top level and tests ;;;;;;;;;;;;;;;;

(deftype run (-> (string) Expressed-Value))
(define run
  (lambda (string)
    (eval-program (scan&parse string))))

(deftype run-all (-> () void))
(define run-all
  (lambda ()
    (run-experiment run use-execution-outcome
      '(lang3-1 lang3-5) all-tests)))

(deftype run-one (-> (symbol) void))
(define run-one
  (lambda (test-name)
    (run-test run test-name)))

;; needed for testing
(deftype equal-external-reps? (-> (datum datum) boolean))
(define equal-external-reps? equal?)

;;;;;;;;;;;;;;;; grammatical specification ;;;;;;;;;;;;;;;;

(define the-lexical-spec
  '((whitespace (whitespace) skip)
    (comment ("%" (arbno (not #\newline))) skip)
    (identifier
      (letter (arbno (or letter digit "_" "-" "?")))
      symbol)
    (number (digit (arbno digit)) number)))

(define the-grammar
  '((program (expression) a-program)

    (expression (number) lit-exp)
    (expression (identifier) var-exp)   
    (expression
      (primitive "(" (separated-list expression ",") ")")
      primapp-exp)
    (expression
      ("if" expression "then" expression "else" expression)
      if-exp)
    (expression
      ("let" (arbno  identifier "=" expression) "in" expression)
      let-exp)
    (expression
      ("proc" "(" (separated-list identifier ",") ")" expression)
      proc-exp)
    (expression
      ("(" expression (arbno expression) ")")
      app-exp)
    (expression
      ("begin" expression (arbno ";" expression) "end")
      begin-exp)

    (primitive ("+")     add-prim)
    (primitive ("-")     subtract-prim)
    (primitive ("*")     mult-prim)
    (primitive ("add1")  incr-prim)
    (primitive ("sub1")  decr-prim)
    (primitive ("zero?") zero-test-prim)
    ))

(deftype show-the-datatypes (-> () (list-of datum)))
(define show-the-datatypes
  (lambda ()
    (sllgen:make-define-datatypes the-lexical-spec the-grammar)
    (sllgen:list-define-datatypes the-lexical-spec the-grammar)))

;; The following is generated by calling show-the-datatypes
;; (and then removing the outer lists, indenting, and renaming fields).
(define-datatype
  program
  program?
  (a-program (exp expression?)))

(define-datatype
  expression
  expression?
  (lit-exp (datum number?))
  (var-exp (id symbol?))
  (primapp-exp
   (prim primitive?)
   (rands (list-of expression?)))
  (if-exp
   (test-exp expression?)
   (true-exp expression?)
   (false-exp expression?))
  (let-exp
   (ids (list-of symbol?))
   (rands (list-of expression?))
   (body expression?))
  (proc-exp
   (ids (list-of symbol?))
   (body expression?))
  (app-exp
   (rator expression?)
   (rands (list-of expression?)))
  (begin-exp
   (first expression?)
   (rest (list-of expression?))))

(define-datatype
  primitive
  primitive?
  (add-prim)
  (subtract-prim)
  (mult-prim)
  (incr-prim)
  (decr-prim)
  (zero-test-prim))

(deftype scan&parse (-> (string) program))
(define scan&parse
  (sllgen:make-string-parser the-lexical-spec the-grammar))

(deftype just-scan (forall (TOK) (-> (string) (list-of TOK))))
(define just-scan
  (sllgen:make-string-scanner the-lexical-spec the-grammar))

(deftype read-eval-print (-> () poof))
(define read-eval-print 
  (sllgen:make-rep-loop  "--> "
    (lambda (pgm) (eval-program pgm))   ; wrapped to avoid load
                                        ; dependency 
    (sllgen:make-stream-parser 
      the-lexical-spec
      the-grammar)))

;;; Domains for this interpreter:
;;;
;;; Expressed-Value = Number + ProcText + List(Expressed-Value)
;;; ProcText = (-> ((list-of Expressed-Value) environment) Expressed-Value)
;;; Denoted-Value = Expressed-Value

;;; ADTs used

(require (lib "environment-as-ribcage.scm" "lib342"))

;;;;;;;;;;;;;;;; procedure texts ;;;;;;;;;;;;;;;;

(deftype proctext? (type-predicate-for proctext))
(deftype text
  (-> ((list-of symbol) expression)
      (-> ((list-of Expressed-Value) environment) Expressed-Value)))
(deftype apply-proctext
  (-> ((-> ((list-of Expressed-Value) environment) Expressed-Value)
       (list-of Expressed-Value)
       environment)
      Expressed-Value))

(define proctext? procedure?)

(define text
  (lambda (ids body)
    (lambda (args env)
        ;; <put your code here>
      )))

(define apply-proctext
  (lambda (proc args env)
    ;; <put your code here>
    ))

;;; Anonymous figure : page 80

(deftype true-value? (-> (Expressed-Value) boolean))
(define true-value?
  (lambda (x)
    (not (zero? (expressed->number x)))))

;;;;;;;;; Expressed-Value ;;;;;;;;;;;;;;;;;;;;;;;;;

;; upcasts
(deftype number->expressed (-> (number) Expressed-Value))
(deftype proctext->expressed
  (-> ((-> ((list-of Expressed-Value) environment) Expressed-Value))
      Expressed-Value))
(deftype list->expressed (-> ((list-of Expressed-Value)) Expressed-Value))

;; downcasts
(deftype expressed->number (-> (Expressed-Value) number))
(deftype expressed->proctext
  (-> (Expressed-Value)
      (-> ((List-Of Expressed-Value) Environment) Expressed-Value)))
(deftype expressed->list (-> (Expressed-Value) (list-of Expressed-Value)))

(require (lib "inject-extract.scm" "lib342"))

; (defrep (Expressed-Value datum))

(define number->expressed
  inject)

(define proctext->expressed
  inject)

(define list->expressed
  inject)

(define expressed-value:extract
  (extract-maker "expressed"))

(define expressed->number
  (expressed-value:extract number? "number"))

(define expressed->proctext
  (expressed-value:extract (lambda (x) (proctext? x)) "proctext"))

(define expressed->list
  (expressed-value:extract list? "list"))


;;;;;;;;;;;;;;;; the interpreter ;;;;;;;;;;;;;;;;

(deftype eval-program (-> (program) Expressed-Value))
(define eval-program 
  (lambda (pgm)
    (cases program pgm
      (a-program (body)
        (eval-expression body (init-env))))))

(deftype eval-expression (-> (expression environment) Expressed-Value))
(define eval-expression 
  (lambda (exp env)
    (cases expression exp
      (lit-exp (datum)
	(number->expressed datum))
      (var-exp (id)
	(apply-env env id))
      (primapp-exp (prim rands)
        (let ((args (eval-rands rands env)))
          (apply-primitive prim args)))
      (if-exp (test-exp true-exp false-exp)
        (if (true-value? (eval-expression test-exp env))
          (eval-expression true-exp env)
          (eval-expression false-exp env)))
      (begin-exp (exp1 exps)
        (let loop ((acc (eval-expression exp1 env))
                   (exps exps))
          (if (null? exps) acc
            (loop (eval-expression (car exps) env) (cdr exps)))))
      (let-exp (ids rands body)
        (let ((args (eval-rands rands env)))
          (eval-expression
	   body
	   (extend-env ids args env))))
      (proc-exp (ids body)
	 ;; exercise 3.30
	 ;; <put your code here>
	 )
      (app-exp (rator rands)
	 ;; exercise 3.30
	 ;; <put your code here>
	 )
      (else (eopl:error 'eval-expression "Missing case:~s" exp))
      )))

(deftype eval-rands
  (-> ((list-of expression) environment) (list-of Expressed-Value)))
(define eval-rands
  (lambda (rands env)
    (map (lambda (x) (eval-rand x env)) rands)))

(deftype eval-rand (-> (expression environment) Expressed-Value))
(define eval-rand
  (lambda (rand env)
    (eval-expression rand env)))

(deftype apply-primitive
  (-> (primitive (list-of Expressed-Value)) Expressed-Value))
(define apply-primitive
  (lambda (prim args)
    (cases primitive prim
      (add-prim  () (number->expressed
		     (+ (expressed->number (car args))
			(expressed->number (cadr args)))))
      (subtract-prim () (number->expressed
			 (- (expressed->number (car args))
			    (expressed->number (cadr args)))))
      (mult-prim  () (number->expressed
		      (* (expressed->number (car args))
			 (expressed->number (cadr args)))))
      (incr-prim  () (number->expressed
		      (+ (expressed->number (car args)) 1)))
      (decr-prim  () (number->expressed
		      (- (expressed->number (car args)) 1)))
      (zero-test-prim () (number->expressed
			  (if (zero? (expressed->number (car args))) 1 0)))
      )))

(deftype init-env (-> () environment))
(define init-env 
  (lambda ()
    (extend-env
      '(i v x)
      (map number->expressed '(1 5 10))
      (empty-env))))
